@@ -2,21 +2,24 @@ package Mojo::PDF;
 
 use Mojo::Base -base;
 
-our $VERSION = '1.002001'; # VERSION
+our $VERSION = '1.003001'; # VERSION
 
 use Carp qw/croak/;
 $Carp::Internal{ (__PACKAGE__) }++;
 use PDF::Reuse 0.36;
-use Graphics::Color::RGB;
+use Number::RGB 1.4;
 use List::AllUtils qw/sum/;
 use Mojo::PDF::Primitive::Table;
 use namespace::clean;
 
 $SIG{'__WARN__'} = sub { warn @_ unless caller eq 'PDF::Reuse'; };
 
-has [qw/_x _y _line_height/] => 0;
-has [qw/_cur_font  _cur_size  _cur_color/];
+has [qw/_line_height  _x  _y /] => 0;
+has _cur_color => sub { [0, 0, 0] };
+has _cur_font  => 'TR';
+has _cur_size  => 12;
 has _fonts => sub { +{} };
+has _rules => sub { +{} };
 
 my $PAGE_SIZE_Y = 792;
 my $PAGE_SIZE_X = 612;
@@ -35,38 +38,56 @@ my %STD_FONTS = (
     'Helvetica-BoldOblique' => 'Helvetica-BoldOblique',
     'Symbol'                => 'Symbol',
     'ZapfDingbats'          => 'ZapfDingbats',
-    'TR'  => 'Times-Roman',
-    'TB'  => 'Times-Bold',
-    'TI'  => 'Times-Italic',
-    'TBI' => 'Times-BoldItalic',
-    'C'   => 'Courier',
-    'CB'  => 'Courier-Bold',
-    'CO'  => 'Courier-Oblique',
-    'CBO' => 'Courier-BoldOblique',
-    'H'   => 'Helvetica',
-    'HB'  => 'Helvetica-Bold',
-    'HO'  => 'Helvetica-Oblique',
-    'HBO' => 'Helvetica-BoldOblique',
-    'S'   => 'Symbol',
-    'Z'   => 'ZapfDingbats'
+    'TR'                    => 'Times-Roman',
+    'TB'                    => 'Times-Bold',
+    'TI'                    => 'Times-Italic',
+    'TBI'                   => 'Times-BoldItalic',
+    'C'                     => 'Courier',
+    'CB'                    => 'Courier-Bold',
+    'CO'                    => 'Courier-Oblique',
+    'CBO'                   => 'Courier-BoldOblique',
+    'H'                     => 'Helvetica',
+    'HB'                    => 'Helvetica-Bold',
+    'HO'                    => 'Helvetica-Oblique',
+    'HBO'                   => 'Helvetica-BoldOblique',
+    'S'                     => 'Symbol',
+    'Z'                     => 'ZapfDingbats'
 );
 
-sub new {
-    my ( $class, $filename ) = @_;
-    my $self = bless {}, $class;
+sub __hex2rgb {
+    my $hex = shift;
+    my $c = eval { Number::RGB->new( hex => $hex ) }
+        or croak "Could not interpret color '$hex' as hex";
 
-    prFile $filename;
-    # use US-Letter pages
-    prMbox ( 0, 0, $PAGE_SIZE_X, $PAGE_SIZE_Y );
-    $self->size;
+    return map $_/255, @{ $c->rgb };
+}
+
+sub __inv_y { $_ = $PAGE_SIZE_Y - $_ for @_; @_ }
+
+sub _line {
+    my $self = shift;
+    my ( $x1, $y1, $x2, $y2 ) = @_;
+    __inv_y($y1, $y2);
+    prAdd "$x1 $y1 m $x2 $y2 l S";
 
     $self;
 }
 
-sub mixin {
+sub _str_width {
     my $self = shift;
-    my ( $doc, $page ) = @_;
-    prForm { file => $doc, page => $page//1 };
+    my $str = shift;
+
+    return prStrWidth(
+        $str,
+        $self->_cur_font//'Helvetica',
+        $self->_cur_size//12
+    ) // 0;
+}
+
+sub _stroke {
+    my $self = shift;
+    my $weight = shift;
+    prAdd "$weight w";
 
     $self;
 }
@@ -84,30 +105,6 @@ sub add_fonts {
     $self;
 }
 
-sub font {
-    my $self = shift;
-    my $name = shift;
-
-    $STD_FONTS{$name} or $self->_fonts->{$name}
-        or croak "Unknown font '$name'";
-
-    $STD_FONTS{$name} ? prFont($name) : prTTFont( $self->_fonts->{$name} );
-    $self->_cur_font($name);
-
-    $self;
-}
-
-sub size {
-    my $self = shift;
-    my $size = shift // 12;
-
-    $self->_cur_size( $size );
-    prFontSize $size;
-    $self->_line_height( $size*1.4 );
-
-    $self;
-}
-
 sub color {
     my $self = shift;
     @_ = @{$_[0]} if @_ == 1 and ref $_[0] eq 'ARRAY';
@@ -124,49 +121,40 @@ sub color {
     $self;
 }
 
-sub __hex2rgb {
-    my $hex = shift;
-    my $c = Graphics::Color::RGB->from_hex_string( $hex )
-        or croak "Could not interpret color '$hex' as hex";
-
-    return $c->as_array;
+sub end {
+    my $self = shift;
+    prEnd;
 }
 
-sub table {
+sub font {
     my $self = shift;
-    my %conf = @_;
+    my $name = shift;
 
-    $conf{row_height} ||= $self->_line_height;
+    $STD_FONTS{$name} or $self->_fonts->{$name}
+        or croak "Unknown font '$name'";
 
-    my $t = Mojo::PDF::Primitive::Table->new(
-        pdf => $self,
-        %conf,
-    );
-
-    my @overflow = $t->draw;
-    return \@overflow if $conf{max_height};
+    $STD_FONTS{$name} ? prFont($name) : prTTFont( $self->_fonts->{$name} );
+    $self->_cur_font($name);
 
     $self;
 }
 
-sub _stroke {
+sub mixin {
     my $self = shift;
-    my $weight = shift;
-    prAdd "$weight w";
+    my ( $doc, $page ) = @_;
+    prForm { file => $doc, page => $page//1 };
 
     $self;
 }
 
-sub text {
-    my $self = shift;
-    my ( $string, $x, $y, $align, $rotation ) = @_;
+sub new {
+    my ( $class, $filename ) = @_;
+    my $self = bless {}, $class;
 
-    $x //= $self->_x;
-
-    # Don't switch to new line, if neither X nor Y were given;
-    $y //= $self->_y + ( @_ > 1 ? $self->_line_height : 0 );
-    $self->_y( $y );
-    $self->_x( (prText($x, __inv_y($y), $string, $align, $rotation))[1] );
+    prFile $filename;
+    # use US-Letter pages
+    prMbox ( 0, 0, $PAGE_SIZE_X, $PAGE_SIZE_Y );
+    $self->size;
 
     $self;
 }
@@ -178,31 +166,107 @@ sub page {
     $self;
 }
 
-sub end {
-    my $self = shift;
-    prEnd;
-}
+sub rule {
+    my ( $self, %new ) = @_;
+    my $rules = $self->_rules;
 
-sub __inv_y { $_ = $PAGE_SIZE_Y - $_ for @_; $_[0] }
-
-sub _line {
-    my $self = shift;
-    my ( $x1, $y1, $x2, $y2 ) = @_;
-    __inv_y($y1, $y2);
-    prAdd "$x1 $y1 m $x2 $y2 l S";
+    for ( keys %new ) {
+        $rules->{$_} = $new{$_};
+        delete $rules->{$_} unless defined $rules->{$_};
+    }
 
     $self;
 }
 
-sub _str_width {
+sub size {
     my $self = shift;
-    my $str = shift;
+    my $size = shift // 12;
 
-    return prStrWidth(
-        $str,
-        $self->_cur_font//'Helvetica',
-        $self->_cur_size//12
-    ) // 0;
+    $self->_cur_size( $size );
+    prFontSize $size;
+    $self->_line_height( $size*1.4 );
+
+    $self;
+}
+
+sub table {
+    my $self = shift;
+    my %conf = @_;
+
+    DRAW: {
+        $conf{row_height} ||= $self->_line_height;
+
+        my $t = Mojo::PDF::Primitive::Table->new( pdf => $self, %conf );
+        my @overflow = $t->draw
+            and $conf{max_height}
+            or return $self;
+
+        $conf{data} = $conf{max_height}->[1]->(\@overflow, \%conf, $self);
+        @{ $conf{data} || [] }
+            and redo DRAW;
+    }
+
+    $self;
+}
+
+sub text {
+    my $self = shift;
+    my ( $string, $x, $y, $align, $rotation ) = @_;
+    $self->_x( $x //= $self->_x );
+
+    # Don't switch to new line, if neither X nor Y were given;
+    $y //= $self->_y + ( @_ > 1 ? $self->_line_height : 0 );
+    $self->_y( $y );
+
+    my @text = ( { text => $string } );
+    for my $r ( values %{$self->_rules} ) {
+        my @new_text;
+        for my $bit ( @text ) {
+            if ( $bit->{marked} ) { push @new_text, $bit; next; }
+
+            my $mark = 0;
+
+            for ( split /($r->{re})/, $bit->{text} ) {
+                if ( $_ =~ /$r->{re}/ ) { $mark = 1; next; }
+                if ( $mark ) {
+                    $mark = 0;
+                    push @new_text, { marked => 1, text => $_, %$r };
+                    next;
+                }
+
+                push @new_text, { text => $_ };
+            }
+        }
+
+        @text = @new_text;
+    }
+
+    my ( $orig_font, $orig_size, $orig_color )
+        = map $self->$_, qw/_cur_font  _cur_size  _cur_color/;
+
+    for my $bit ( @text ) {
+        if ( $bit->{marked} ) {
+            if ( $bit->{font} )  { $self->font(  $bit->{font}  ); }
+            if ( $bit->{size} )  { $self->size(  $bit->{size}  ); }
+            if ( $bit->{color} ) { $self->color( $bit->{color} ); }
+            $self->_text($bit->{text}, $self->_x, $self->_y,$align,$rotation );
+            if ( $bit->{font} )  { $self->font(  $orig_font  ); }
+            if ( $bit->{size} )  { $self->size(  $orig_size  ); }
+            if ( $bit->{color} ) { $self->color( $orig_color ); }
+            next;
+        }
+        $self->_text( $bit->{text}, $self->_x, $self->_y, $align, $rotation );
+    }
+
+    $self;
+}
+
+sub _text {
+    my $self = shift;
+    my ( $string, $x, $y, $align, $rotation, $mods ) = @_;
+    $self->_x( (prText($x, __inv_y($y), $string, $align, $rotation))[1] );
+
+    $self;
 }
 
 q|
@@ -243,9 +307,9 @@ Mojo::PDF - Generate PDFs with the goodness of Mojo!
             galaxie_it => 'fonts/GalaxiePolaris-BookItalic.ttf',
         )
         ->font('galaxie')->size(24)->color('#353C8C')
-            ->text( 'Weeee', 20.4, 75 )
-            ->text( 'eeee continuing same line!')
-            ->text( 'Started a new line!', 20.4 )
+            ->text('Weeee', 20.4, 75 )
+            ->text('eeee continuing same line!')
+            ->text('Started a new line!', 20.4 )
 
         # Render a table
         ->font('galaxie_it')->size(8)->color
@@ -339,6 +403,25 @@ which defaults to the first page.
 
 Add a new blank page to your document and sets it as the currently active page.
 
+=head2 C<rule>
+
+    ->rule(
+        bold  => { re => qr/\*\*(.?)\*\*/, font => 'galaxie_bold' },
+        shiny => {
+            re    => qr/!!(.?)!!/,
+            font  => 'galaxie_bold',
+            color => '#FBBC05',
+            size  => 30,
+        },
+    )
+    ->text('Normal **bold text** lalalala !!LOOK SHINY!!')
+    ->rule( shiny => undef )
+    ->text('!!no longer shiny!!')
+
+Sets rules for bits of text when rendering with
+L</text> or L</table>. Available overrides are L</font>, L</color>,
+and C</size>. To disable a rule, set its value to C<undef>.
+
 =head2 C<size>
 
     $pdf->size(24);
@@ -359,7 +442,12 @@ Specifies active font size in points. Defaults to C<12> points.
         #Optional:
         border         => [.5, '#CFE3EF'],
         header         => 'galaxie_bold',
-        max_height     => 744,
+        max_height     => [ 744, sub {
+            my ( $data, $conf, $pdf ) = @_;
+            $conf->{at}[1] = 50;
+            $pdf->page;
+            $data;
+        },
         min_width      => 571.2,
         row_height     => 24,
         str_width_mult => 1.1,
@@ -402,20 +490,24 @@ C<header> font. B<Not set by default.>
 
 =head3 C<max_height>
 
-    {
-        $data = $pdf->table(
-            max_height => 744,
-            data       => $data,
-            at         => [20.4, 50],
-        );
+    $pdf->table(
+        at         => [20.4, 300],
+        data       => $data,
+        max_height => [ 744, sub {
+            my ( $data, $conf, $pdf ) = @_;
+            $conf->{at}[1] = 50; # start table higher on subsequent pages
+            $pdf->page;          # start a new page
+            $data;               # render remaining rows
+        },
+    );
 
-        @$data and $pdf->page and redo;
-    }
-
-B<Optional>. B<ALTERS RETURN VALUE OF C<table> METHOD>. Specifies the maximum
-height of the table. Specifying this option makes C<table> method return
-an arrayref of rows that did not fit into the table. You can use that to
-add a new page and continue rendering the overflown data.
+B<Optional>. Takes an arrayref with two arguments: the maximum height
+(in points) the table should reach and the callback to use when not
+all rows could fit. The B<return value> of the callback will be used as
+the new collection of rows to render.
+The C<@_> will contain remaining rows to render,
+hashref of the options you've passed to C<table> method, and the
+C<Mojo::PDF> object.
 
 =head3 C<min_width>
 
